from __future__ import annotations

import requests

from enum import Enum
from tqdm import tqdm
from semantic_version import Version
from typing import List, Dict
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .dl_result import DownloadResult
from ..core.search_result import SearchResult
from ..core.loaders import Loaders


curse_base = "https://addons-ecs.forgesvc.net/api/v2/addon"
modrinth_base = "https://api.modrinth.com/api/v1"

_headers = {
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.77 Safari/537.36"
}


class API:
    def __init__(self, base_url: str):
        self._base_url = base_url

    def search(self, search_term: str) -> List[SearchResult]:
        raise NotImplementedError

    def download(self, slug: str, version: Version, loader: Loaders) -> DownloadResult:
        raise NotImplementedError

    @staticmethod
    def get_json(url: str) -> Dict:
        return requests.get(url, headers=_headers).json()


def parse_versions(game_versions: List[str]) -> List[Version]:
    versions = []
    for version in game_versions:
        try:
            versions.append(Version.coerce(version))
        except ValueError:
            continue

    return versions


def get_file(url: str, filename: str, with_progress: bool = False) -> None:
    r = requests.get(url, stream=with_progress)
    with open(filename, "wb") as f:
        if with_progress:
            pbar = tqdm(unit="B", total=int(r.headers["Content-Length"]))
            for chunk in r.iter_content(chunk_size=1024):
                if chunk:  # filter out keep-alive new chunks
                    pbar.update(len(chunk))
                    f.write(chunk)
        else:
            f.write(r.content)


class Sites(Enum):
    Curseforge = "CurseForge"
    Modrinth = "Modrinth"



from __future__ import annotations

from typing import List
from columnar import columnar
from termcolor import colored, cprint

from ..core.search_result import SearchResult
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..remote.dl_result import DownloadResult


def print_search_results(search_results: List[SearchResult]) -> None:
    cols = [
        colored("Name", attrs=["bold"]),
        colored("Slug", attrs=["bold"]),
        colored("Description", attrs=["bold"]),
        colored("Last Updated", attrs=["bold"]),
        colored("Supported Loaders", attrs=["bold"]),
    ]

    table = columnar(
        data=[result.get_row() for result in search_results],
        headers=cols,
        no_borders=False,
    )

    print(table)


def print_installed_mods(local_mods: List[DownloadResult]) -> None:
    cols = [
        colored("Slug", attrs=["bold"]),
        colored("Last Updated", attrs=["bold"]),
        colored("Source", attrs=["bold"]),
    ]

    table = columnar(
        data=[mod.get_row() for mod in local_mods], headers=cols, no_borders=False
    )

    print(table)

    info(f"{len(local_mods)} mods currently installed")


def print_installed_slugs(local_mods: List[str]) -> None:
    print(*local_mods)


def info(msg: str) -> None:
    cprint(f" {msg}", color="cyan")


def success(msg: str) -> None:
    cprint(f" {msg}", color="green")


def warning(msg: str) -> None:
    cprint(f"⚠️ {msg}", color="yellow")


def error(msg: str) -> None:
    cprint(f" {msg}", color="red")



from __future__ import annotations

from enum import Enum


class Loaders(Enum):
    Forge = "forge"
    Fabric = "fabric"
    Unknown = "unknown"

    @staticmethod
    def from_str(enum_string: str) -> Loaders:
        if enum_string.lower() == "forge":
            return Loaders.Forge
        if enum_string.lower() == "fabric":
            return Loaders.Fabric
        return Loaders.Unknown



import json

from pathlib import Path
from typing import Optional
from zipfile import ZipFile

from ...remote.dl_result import DownloadResult
from ...core.loaders import Loaders

_fabric_file = "fabric.mod.json"
_forge_file = "META-INF/mods.toml"


def parse_jar(jar_file: Path) -> Optional[DownloadResult]:
    jar_type = _jar_type(jar_file)

    with ZipFile(jar_file, "r") as unzipped_jar:
        if jar_type == Loaders.Fabric:
            return _parse_fabric(unzipped_jar)

        if jar_type == Loaders.Forge:
            return _parse_force(unzipped_jar)


def _parse_fabric(file: ZipFile) -> DownloadResult:
    with file.open(_fabric_file) as json_file:
        full_doc = json_file.read().decode("utf-8", "ignore")
        mod_data = json.loads(full_doc, strict=False)


def _parse_force(file: ZipFile) -> DownloadResult:
    pass


def _jar_type(jar_file: Path) -> Loaders:
    with ZipFile(jar_file, "r") as f:
        if _is_forge(f):
            return Loaders.Forge
        if _is_fabric(f):
            return Loaders.Fabric
    return Loaders.Unknown


def _is_fabric(zip_file: ZipFile) -> bool:
    return _fabric_file in zip_file.namelist()


def _is_forge(zip_file: ZipFile) -> bool:
    return _forge_file in zip_file.namelist()



from __future__ import annotations

import arrow

from typing import List, Dict
from termcolor import colored

from .loaders import Loaders


class SearchResult:
    def __init__(
        self,
        name: str,
        slug: str,
        desc: str,
        updated: str,
        modloaders: List[str],
        mod_id: str,
    ):
        self.name = name
        self.slug = slug
        self.desc = desc
        self.updated = arrow.get(updated)
        self.modloaders = [
            loader
            for loader in map(Loaders.from_str, modloaders)
            if loader != Loaders.Unknown
        ]
        self.id = mod_id

    def get_row(self) -> List:
        return [
            colored(self.name, color="green"),
            self.slug,
            self.desc,
            self.updated.format("YYYY-MM-DD"),
            ", ".join(loader.name for loader in self.modloaders),
        ]

    @staticmethod
    def is_valid(api_data: Dict) -> bool:
        keys = ["name", "summary", "dateModified", "modLoaders", "id"]

        return all(key in api_data for key in keys)

    def __repr__(self):
        return self.name



from typing import List
from semantic_version import Version

from . import api
from .curse_api import CurseAPI
from .dl_result import DownloadResult
from .modrinth_api import ModrinthAPI
from ..core.exceptions import ModNotFoundException, VersionNotFoundException
from ..core.loaders import Loaders
from ..core.search_result import SearchResult


class Gateway:
    def __init__(self):
        self.curse_api = CurseAPI(api.curse_base)
        self.modrinth_api = ModrinthAPI(api.modrinth_base)

    def search(self, search_term) -> List[SearchResult]:
        curse_results = self.curse_api.search(search_term)
        modrinth_results = self.modrinth_api.search(search_term)

        return sorted((*curse_results, *modrinth_results), key=lambda x: x.name)

    def download(self, slug: str, version: Version, loader: Loaders) -> DownloadResult:
        try:
            return self.curse_api.download(slug, version, loader)
        except (ModNotFoundException, VersionNotFoundException):
            return self.modrinth_api.download(slug, version, loader)



import os

import fire
from semantic_version import Version

from pacmine.core.exceptions import ModNotFoundException, VersionNotFoundException
from pacmine.core.loaders import Loaders
from pacmine.local import log
from pacmine.local.serialize.serializer import Serializer
from pacmine.remote.gateway import Gateway

LOCATION = os.getcwd()


def add(*slugs: str) -> None:
    repo = Serializer(LOCATION)
    gateway = Gateway()

    if not repo.is_valid():
        log.error("Not a valid mod repository")
        return

    for slug in slugs:
        if repo.is_installed(slug):
            log.info(f"{slug} is already installed")
            continue

        try:
            result = gateway.download(slug, *repo.get_metadata())
            repo.add(result)
            log.success(f"Installed {slug}")
        except ModNotFoundException:
            log.error(f"{slug} could not be found")
        except VersionNotFoundException:
            log.error(f"No suitable version found for {slug}")


def remove(*slugs: str) -> None:
    repo = Serializer(LOCATION)

    if not repo.is_valid():
        log.error("Not a valid mod repository")
        return

    for slug in slugs:
        try:
            repo.remove(slug)
            log.success(f"Removed {slug}")
        except ModNotFoundException:
            log.info(f"{slug} is not installed")


def search(search_term: str) -> None:
    gateway = Gateway()

    results = gateway.search(search_term)

    if not results:
        log.error(f'No results found for "{search_term}"')
        return

    log.print_search_results(results)


def list() -> None:
    repo = Serializer(LOCATION)

    if not repo.is_valid():
        log.error("Not a valid mod repository")
        return

    installed_mods = repo.get_installed_mods()

    if not installed_mods:
        log.info("No mods installed")
        return

    log.print_installed_mods(installed_mods)


def dump() -> None:
    repo = Serializer(LOCATION)

    if not repo.is_valid():
        log.error("Not a valid mod repository")
        return

    installed_slugs = repo.get_installed_slugs()

    log.print_installed_slugs(installed_slugs)


def new(version_str: float, loader_str: str) -> None:
    repo = Serializer(LOCATION)

    if repo.is_valid():
        log.warning(
            f"Mod repository already exists in {LOCATION}. Delete mods.pkl and try again."
        )
        return

    try:
        version = Version.coerce(str(version_str))
    except ValueError:
        log.error(f"Invalid version: {version_str}")
        return

    loader = Loaders.from_str(loader_str.lower())
    if loader == Loaders.Unknown:
        log.error(f"Unknown modloader: {loader_str}")
        return

    repo.new(version, loader)


def main():
    fire.Fire(
        {
            "add": add,
            "remove": remove,
            "search": search,
            "list": list,
            "dump": dump,
            "new": new,
        }
    )


if __name__ == "__main__":
    main()



import os
import pickle
from typing import List, Tuple
from semantic_version import Version

from .repo_state import State
from ...core.exceptions import ModNotFoundException
from ...core.loaders import Loaders
from ...local.persistence import Persistor
from ...remote.dl_result import DownloadResult


FILE_NAME = "mods.pkl"


class Serializer(Persistor):
    def get_metadata(self) -> Tuple[Version, Loaders]:
        state = self._read_pickle()
        return state.version, state.loader

    def is_valid(self) -> bool:
        if os.path.exists(FILE_NAME):
            state = self._read_pickle()
            return state.is_valid()

    def new(self, version: Version, loader: Loaders) -> None:
        state = State(version, loader)
        self._write_pickle(state)

    def get_installed_mods(self) -> List[DownloadResult]:
        state = self._read_pickle()

        return sorted(state.installed_mods, key=lambda x: x.slug)

    def get_installed_slugs(self) -> List[str]:
        state = self._read_pickle()

        return _get_slugs(state)

    def is_installed(self, slug: str) -> bool:
        state = self._read_pickle()

        return slug in _get_slugs(state)

    def add(self, mod: DownloadResult) -> None:
        state = self._read_pickle()

        if mod not in state.installed_mods:
            state.installed_mods.append(mod)

        self._write_pickle(state)

    def remove(self, slug: str) -> None:
        state = self._read_pickle()

        if slug not in _get_slugs(state):
            raise ModNotFoundException

        for mod in state.installed_mods:
            if mod.slug == slug:
                os.remove(mod.filename)
                state.installed_mods.remove(mod)
                break

        self._write_pickle(state)

    def _read_pickle(self) -> State:
        with open(self.location.joinpath(FILE_NAME), "rb") as infile:
            return pickle.load(infile)

    def _write_pickle(self, state: State) -> None:
        with open(self.location.joinpath(FILE_NAME), "wb") as outfile:
            pickle.dump(state, outfile)


def _get_slugs(state: State) -> List[str]:
    return [mod.slug for mod in state.installed_mods]



from pathlib import Path
from typing import List, Tuple

from semantic_version import Version

from ..core.loaders import Loaders
from ..remote.dl_result import DownloadResult


class Persistor:
    def __init__(self, location: str):
        self.location = Path(location)

    def is_valid(self) -> bool:
        raise NotImplementedError

    def new(self, version: Version, loader: Loaders) -> None:
        raise NotImplementedError

    def get_installed_mods(self) -> List[DownloadResult]:
        raise NotImplementedError

    def get_installed_slugs(self) -> List[str]:
        raise NotImplementedError

    def is_installed(self, slug: DownloadResult) -> bool:
        raise NotImplementedError

    def add(self, mod: DownloadResult) -> None:
        raise NotImplementedError

    def remove(self, slug: str) -> None:
        raise NotImplementedError

    def get_metadata(self) -> Tuple[Version, Loaders]:
        raise NotImplementedError



from typing import List
from semantic_version import Version

from ...core.loaders import Loaders
from ...remote.dl_result import DownloadResult


class State:
    def __init__(self, version: Version, loader: Loaders):
        self.version = version
        self.loader = loader
        self.installed_mods: List[DownloadResult] = []

    def is_valid(self) -> bool:
        return self.version is not None and self.loader is not None



from __future__ import annotations

import arrow

from typing import List

from .api import Sites


class DownloadResult:
    def __init__(
        self, slug: str, mod_id: str, date_updated: str, filename: str, source: Sites
    ):
        self.slug = slug
        self.mod_id = mod_id
        self.date_updated = arrow.get(date_updated)
        self.filename = filename
        self.source = source

    def get_row(self) -> List[str]:
        return [self.slug, self.date_updated.format("YYYY-MM-DD"), self.source.value]

    def __eq__(self, other: DownloadResult) -> bool:
        return (
            self.slug == other.slug
            and self.mod_id == other.mod_id
            and self.date_updated == other.date_updated
            and self.filename == other.filename
            and self.source == other.source
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pacmine',
 'pacmine.core',
 'pacmine.local',
 'pacmine.local.discovery',
 'pacmine.local.serialize',
 'pacmine.remote']

package_data = \
{'': ['*']}

install_requires = \
['Columnar>=1.3.1,<2.0.0',
 'arrow>=1.1.1,<2.0.0',
 'fire>=0.4.0,<0.5.0',
 'requests>=2.26.0,<3.0.0',
 'semantic-version>=2.8.5,<3.0.0',
 'termcolor>=1.1.0,<2.0.0',
 'tqdm>=4.62.0,<5.0.0']

entry_points = \
{'console_scripts': ['pacmine = pacmine.__main__:main']}

setup_kwargs = {
    'name': 'pacmine',
    'version': '1.0.0',
    'description': '',
    'long_description': None,
    'author': 'aidan',
    'author_email': 'aidan.chaplin@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from typing import List, Dict

import arrow
from semantic_version import Version

from . import api
from .dl_result import DownloadResult
from ..core.exceptions import ModNotFoundException, VersionNotFoundException
from ..core.loaders import Loaders
from ..core.search_result import SearchResult
from ..remote.api import API, Sites


class ModrinthAPI(API):
    def search(self, search_term: str) -> List[SearchResult]:
        results = API.get_json(self._make_search_url(search_term))

        if "hits" not in results:
            return []

        mods = []
        for mod in results["hits"]:
            mod_id = mod["mod_id"].replace("local-", "")
            name = mod["title"]
            slug = mod["slug"]
            desc = mod["description"]
            updated = mod["date_modified"]
            modloaders = mod["categories"]

            mods.append(SearchResult(name, slug, desc, updated, modloaders, mod_id))

        return mods

    def download(self, slug: str, version: Version, loader: Loaders) -> DownloadResult:
        mod_id = self._slug_to_id(slug)

        download_info = self._get_best_download(mod_id, version, loader)

        date_uploaded = download_info["date_published"]
        upload_link = download_info["files"][0]["url"]
        file_name = download_info["files"][0]["filename"]

        api.get_file(upload_link, file_name)

        return DownloadResult(slug, mod_id, date_uploaded, file_name, Sites.Modrinth)

    def _make_search_url(self, search_term: str) -> str:
        return f"{self._base_url}/mod?query={search_term}"

    def _make_files_url(self, mod_id: str) -> str:
        return f"{self._base_url}/mod/{mod_id}/version"

    def _slug_to_id(self, slug: str) -> str:
        mod_results = self.search(slug)

        for result in mod_results:
            if result.slug == slug:
                return result.id
        else:
            raise ModNotFoundException

    def _get_best_download(self, mod_id, version, loader) -> Dict:
        files = API.get_json(self._make_files_url(mod_id))

        valid_loaders = [file for file in files if loader.value in file["loaders"]]

        valid_versions = [
            v_file
            for v_file in valid_loaders
            if version in api.parse_versions(v_file["game_versions"])
        ]

        if not valid_versions:
            raise VersionNotFoundException

        to_download = max(
            valid_versions, key=lambda x: arrow.get(x["date_published"]), default=None
        )

        return to_download
